function J=KBA(C,H,nh,sigma,DD,thr)

% Kernel Bayes Accuracy criterion

% Bayes Optimal Kernel Discriminant Analysis. IEEE conference on Computer
% Vision and Pattern Recognition, 2010. 

% This function is an implementation of the Kernel Bayes Accuracy (KBA)
% criterion. 

% Input: 

% C: number of classes

% H: a 1-by-C vector with each element indicating the number of subclassses
% in each class.

% nh: a 1-by-C*sum(H) vector with each element indicating the number of
% samples in each subclass.

% sigma: the RBF kernel parameter

% DD: the Euclidean distance matrix of the pairwise samples

% Output:

% J: the criterion value

% Copyrighted code
% (c) Di You & Aleix M Martinez
%
% For additional information contact the authors


warning off
opts.disp=0;
K1=exp(-DD/(2*sigma^2));
l=size(K1,2);
M1=1/size(K1,1)*ones(size(K1));
K1=K1-M1*K1-K1*M1+M1*K1*M1; % mean normalize in kernel space

% whiten the kernel matrix in the kernel space using kernel covariance
% matrix. This step is not necessary if the kernel covariance matrix is used.
% Because the whitened kernel matrix is the same as the original mean
% normalized kernel matrix if one goes through the derivations. 
 
% [vk dk]=eigs(K1,rank(K1),'lm',opts);
% Kw=K1*(vk*diag(((diag(dk)).^(-1)))*vk')*K1;

Kw=K1;
HH = sum(H);

start = 0;
MM=zeros(l,HH);


for i=1: HH
    temp = Kw(:,start+1:start+nh(i));
    MM(1:l,i)=mean(temp,2); % obtain kernel mean in each class
    for class = 1:C
        if (i <= sum(H(1:class)))
            sub_label(i) = class;
            break;
        end
    end
    start = sum(nh(1:i));   
end

for i=1:l
    for j = 1:HH
        if (i <= sum(nh(1:j)))
            label(i) = j;
            break;
        end
    end
end

% calculate the sum of pairwise Bayes accuracies
A=zeros(l,l);

 for i=1:HH-1
    for j=i+1:HH
        if (sub_label(i) ~= sub_label(j))
            K_i= Kw(label==i,label==i);
            K_j= Kw(label==j,label==j);
            K_ij=Kw(label==i,label==j);
            K_ji=K_ij';
            D(i,j)=ones(nh(i),1)'/nh(i)*K_i*ones(nh(i),1)/nh(i)-ones(nh(i),1)'/nh(i)*K_ij*ones(nh(j),1)/nh(j) ...
                -ones(nh(j),1)'/nh(j)*K_ji*ones(nh(i),1)/nh(i)+ones(nh(j),1)'/nh(j)*K_j*ones(nh(j),1)/nh(j);
            A=A+(nh(i)/l)*(nh(j)/l)*1/(2*D(i,j))*erf(sqrt(D(i,j))/(2*sqrt(2))) ...
                *((MM(:,i)-MM(:,j))*(MM(:,i)-MM(:,j))');
        end
    end
 end

% keep the top eigenvectors and eigenvalues by deleting the smallest
% eigenvalues which are probably due to the noise. 
 [va da]=eigs(A,rank(A),'lm',opts);
 da=diag(da);
ii=1;
while(sum(da(1:ii))< thr*sum(da))
    ii=ii+1;
end
da(ii+1:end)=[];
va(:,ii+1:end)=[]; 

 J=0;
 for m=1:size(va,2)
  for i=1:HH-1
    for j=i+1:HH
        if (sub_label(i) ~= sub_label(j))
            
            J=J+(nh(i)/l)*(nh(j)/l)*1/(2*D(i,j))*erf(sqrt(D(i,j))/(2*sqrt(2))) ...
                *(va(:,m)'*(MM(:,i)-MM(:,j))*(MM(:,i)-MM(:,j))'*va(:,m));
        end
    end
  end
 end

% minimizing instead of maximizing
J=-J;

